@@ -31,14 +31,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ctcp.c,v 1.51 2006/04/30 14:15:43 f Exp $
+ * $Id: ctcp.c,v 1.56 2021-04-26 19:57:28 t Exp $
  */
 
 #include "irc.h"
 
-#ifndef _Windows
 #include <pwd.h>
-#endif /* _Windows */
 
 #ifdef HAVE_UNAME
 # include <sys/utsname.h>
@@ -89,13 +87,14 @@ extern void AwaySave _((char *, int));
 extern void CheckCdcc _((char *, char *, char *, int));
 extern void ColorUserHost _((char *, char *, char *, int));
 extern char *YNreply _((int));
-extern void EncryptString _((char *, char *, char *, int, int));
+extern void EncryptString _((char *, char *, char *, int, int, int));
 extern void ChannelLogSave _((char *, ChannelList *));
 extern int  AutoReplyMatch _((char *));
 extern void Check4WordKick _((char *, NickList *, int, ChannelList *));
+extern void dcc_reject _((char *, char *, char *));
 /****************************************************************************/
 
-static	char	FAR CTCP_Reply_Buffer[BIG_BUFFER_SIZE + 1] = "";
+static	char	CTCP_Reply_Buffer[BIG_BUFFER_SIZE + 1] = "";
 
 static	void	do_new_notice_ctcp _((char *, char *, char **, char *));
 
@@ -211,7 +210,7 @@ char	*ctcp_type[] =
 	"NOTICE"
 };
 
-static	char	FAR ctcp_buffer[BIG_BUFFER_SIZE + 1];
+static	char	ctcp_buffer[BIG_BUFFER_SIZE + 1];
 
 /* This is set to one if we parsed an SED */
 int     sed = 0;
@@ -337,7 +336,7 @@ static int dropit(int allowtwo) {
     ctcpcount++;
     if (server_list[parsing_server_index].ctcp_last_reply_time+3>time((time_t *) 0)) {
         if (!allowtwo) return(1);
-        if (!ctcpcount>2) return(1);
+        if (ctcpcount>2) return(1);
     }
     if (ctcpcount>2) ctcpcount=1;
     return(0);
@@ -409,16 +408,16 @@ char *nick;
     send_to_server("NOTICE %s :This function has been disabled  -ScrollZ-",nick);
 }
 
-static int checkpassword(tmpfriend,passwd)
+static int checkpassword(tmpfriend, passwd)
 struct friends *tmpfriend;
 char *passwd;
 {
-    char passbuf[mybufsize/8];
+    char passbuf[mybufsize / 8];
 
     if (tmpfriend->passwd) {
         if (passwd && *passwd) {
-            EncryptString(passbuf,passwd,passwd,mybufsize/16,0);
-            return((!strcmp(passbuf,tmpfriend->passwd))?0:1);
+            EncryptString(passbuf, passwd, passwd, mybufsize/16, 0, SZ_ENCR_OTHER);
+            return((!strcmp(passbuf, tmpfriend->passwd)) ? 0 : 1);
         }
         return(1);
     }
@@ -1253,12 +1252,8 @@ do_version(ctcp, from, to, cmd)
                 the_unix = un.sysname;
         }
         send_ctcp_reply(from, ctcp->name, "ircII %s %s %s :%s", irc_version, the_unix, the_version,
-#else
-#ifdef _Windows
-        send_ctcp_reply(from, ctcp->name, "ircII %s MS-Windows :%s", irc_version,
 #else
         send_ctcp_reply(from, ctcp->name, "ircII %s *IX :%s", irc_version,
-#endif /* _Windows */
 #endif /* HAVE_UNAME */
 		(tmp = get_string_var(CLIENTINFO_VAR)) ?  tmp : IRCII_COMMENT);
 #endif /* PARANOID */
@@ -1374,37 +1369,31 @@ do_finger(ctcp, from, to, cmd)
 	diff = time(0) - idle_time;
 	c = (diff == 1) ? ' ' : 's';
 	/* XXX - fix me */
-# ifdef _Windows
-	send_ctcp_reply(from, ctcp->name,
-		"IRCII For MS-Windows User Idle %d second%c",
-		(int)diff, c);
-# else
 	uid = getuid();
-#  ifdef DAEMON_UID
+# ifdef DAEMON_UID
 	if (uid != DAEMON_UID)
 	{
-#  endif /* DAEMON_UID */
+# endif /* DAEMON_UID */
 		if ((pwd = getpwuid(uid)) != NULL)
 		{
 			char	*tmp;
 
-#  ifndef GECOS_DELIMITER
-#   define GECOS_DELIMITER ','
-#  endif /* GECOS_DELIMITER */
+# ifndef GECOS_DELIMITER
+#  define GECOS_DELIMITER ','
+# endif /* GECOS_DELIMITER */
 			if ((tmp = index(pwd->pw_gecos, GECOS_DELIMITER)) != NULL)
 				*tmp = '\0';
 			send_ctcp_reply(from, ctcp->name,
 				"%s (%s@%s) Idle %d second%c", pwd->pw_gecos,
 				pwd->pw_name, hostname, (int)diff, c);
 		}
-#  ifdef DAEMON_UID
+# ifdef DAEMON_UID
 	}
 	else
 		send_ctcp_reply(from, ctcp->name,
 			"IRCII Telnet User (%s) Idle %d second%c",
 			realname, (int)diff, c);
-#  endif /* DAEMON_UID */
-# endif /* _Windows */
+# endif /* DAEMON_UID */
 #endif /* PARANOID */
 /**************************** PATCHED by Flier ******************************/
         }
@@ -1428,8 +1417,8 @@ do_atmosphere(ctcp, from, to, cmd)
 {
 /**************************** PATCHED by Flier ******************************/
         int     foundar = 0;
-		int		isshit;
-		int		isfriend;
+        int     isshit;
+        int     isfriend;
         char    *thing;
 #ifdef WANTANSI
         char    *color = CmdsColors[COLME].color3;
@@ -1437,10 +1426,10 @@ do_atmosphere(ctcp, from, to, cmd)
         char    *color = "";
 #endif
         char    tmpbuf1[mybufsize / 2];
-        void    (*func)();
+        Window *(*func)();
         ChannelList *chan;
 	NickList *joiner;
-        Window *oldwin;
+        Window *oldwin, *w = NULL;
 
 #ifdef HAVE_ICONV_H
         if (get_int_var(HIGH_ASCII_VAR)) thing = "\342\210\236";
@@ -1448,8 +1437,8 @@ do_atmosphere(ctcp, from, to, cmd)
         if (get_int_var(HIGH_ASCII_VAR)) thing = "";
 #endif /* HAVE_ICONV_H */
         else thing = "*";
-        if (Stamp == 2) func = (void(*)()) say;
-        else func = (void(*)()) put_it;
+        if (Stamp == 2) func = (Window *(*)()) say;
+        else func = (Window *(*)()) put_it;
 /****************************************************************************/
         if (cmd && *cmd)
 	{
@@ -1482,12 +1471,12 @@ do_atmosphere(ctcp, from, to, cmd)
 					put_it("* %s:%s %s", from, to, cmd);*/
 				if (is_current_channel(to, parsing_server_index, 0))
 #ifdef WANTANSI
-                                    func("%s%s%s %s%s%s %s%s%s",
-                                          CmdsColors[COLME].color1, thing, Colors[COLOFF],
-                                          color, from, Colors[COLOFF],
-                                          CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
+                                    w = func("%s%s%s %s%s%s %s%s%s",
+                                             CmdsColors[COLME].color1, thing, Colors[COLOFF],
+                                             color, from, Colors[COLOFF],
+                                             CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
 #else
-                                    func("%s %s%s%s %s", thing, color, from, color, cmd);
+                                    w = func("%s %s%s%s %s", thing, color, from, color, cmd);
 #endif
                                 else {
 #ifdef WANTANSI
@@ -1495,10 +1484,10 @@ do_atmosphere(ctcp, from, to, cmd)
                                            CmdsColors[COLME].color4, to, Colors[COLOFF],
                                            CmdsColors[COLME].color1, thing, Colors[COLOFF],
                                            color, from, Colors[COLOFF]);
-                                    func("%s %s%s%s",tmpbuf1,
-                                          CmdsColors[COLME].color5,cmd, Colors[COLOFF]);
+                                    w = func("%s %s%s%s",tmpbuf1,
+                                             CmdsColors[COLME].color5,cmd, Colors[COLOFF]);
 #else
-                                    func("<%s> %s %s%s%s %s", to, thing, color, from, color, cmd);
+                                    w = func("<%s> %s %s%s%s %s", to, thing, color, from, color, cmd);
 #endif
                                 }
 /****************************************************************************/
@@ -1548,10 +1537,10 @@ do_atmosphere(ctcp, from, to, cmd)
                                     to_window = curr_scr_win;
                             }
 #ifdef WANTANSI
-                            func("%s %s%s%s", tmpbuf1,
-                                 CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
+                            w = func("%s %s%s%s", tmpbuf1,
+                                     CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
 #else
-                            func("%s", tmpbuf1);
+                            w = func("%s", tmpbuf1);
 #endif
                             to_window = oldwin;
                         }
@@ -1570,18 +1559,22 @@ do_atmosphere(ctcp, from, to, cmd)
 /**************************** PATCHED by Flier ******************************/
 				/*put_it("*> %s %s", from, cmd);*/
 #ifdef WANTANSI
-                                func("%s%s%s> %s%s%s %s%s%s",
-                                      CmdsColors[COLME].color1, thing, Colors[COLOFF],
-                                      color, from, Colors[COLOFF],
-                                      CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
+                                w = func("%s%s%s> %s%s%s %s%s%s",
+                                         CmdsColors[COLME].color1, thing, Colors[COLOFF],
+                                         color, from, Colors[COLOFF],
+                                         CmdsColors[COLME].color5, cmd, Colors[COLOFF]);
 #else
-				func("%s> %s%s%s %s", thing, color, from, color, cmd);
+				w = func("%s> %s%s%s %s", thing, color, from, color, cmd);
 #endif
 /****************************************************************************/
 		}
 		set_lastlog_msg_level(old);
  		restore_message_from();
 	}
+        if (foundar && w && (w != curr_scr_win)) {
+            w->miscflags |= WINDOW_REPWORD;
+            update_all_status();
+        }
 	return NULL;
 }
 
@@ -1604,9 +1597,20 @@ do_dcc(ctcp, from, to, args)
 
 	if (my_stricmp(to, get_server_nickname(parsing_server_index)))
 		return NULL;
-	if (!(type = next_arg(args, &args)) ||
 /**************************** PATCHED by Flier ******************************/
-			/*!(description = next_arg(args, &args)) ||*/
+        /* handle DCC reject separately */
+	/*if (!(type = next_arg(args, &args)) ||
+			!(description = next_arg(args, &args)) ||*/
+        type = next_arg(args, &args);
+        if (type && !my_stricmp(type, "REJECT")) {
+            char *dcc_type = next_arg(args, &args);
+            char *dcc_file = new_next_arg(args, &args);
+
+            if (dcc_type && dcc_file)
+                dcc_reject(from, dcc_type, dcc_file);
+            return NULL;
+        }
+        else if (!type ||
                         /* support filenames enclosed in quotes */
 			!(description = new_next_arg(args, &args)) ||
 /****************************************************************************/
@@ -1625,14 +1629,29 @@ do_utc(ctcp, from, to, args)
 		*to,
 		*args;
 {
-	time_t	tm;
+	time_t	tm = time(NULL),
+		curtime;
 	char	*date = NULL;
 
 	if (!args || !*args)
 		return NULL;
 	tm = atol(args);
-	malloc_strcpy(&date, ctime(&tm));
-	date[strlen(date)-1] = '\0';
+	curtime = ctime(&tm);
+
+	if (curtime)
+	{
+		u_char *s = index(curtime, '\n');
+		if (s)
+		{
+			*s = '\0';
+		}
+		malloc_strcpy(&date, UP(curtime));
+	}
+	else
+	{
+		/* if we can't find a time, just return the number */
+		malloc_strcpy(&date, args);
+	}
 	return date;
 }
 
@@ -1720,7 +1739,7 @@ do_ctcp(from, to, str)
                         }
 			new_free(&arg_copy);
 /**************************** PATCHED by Flier ******************************/
-                        if (!check_flooding(from,CTCP_FLOOD,args)) continue;
+                        if (!check_flooding(from,to,CTCP_FLOOD,args)) continue;
                         /* if it came via DCC CHAT only allow ACTION to be processed */
                         if (*from=='=' && strcmp(cmd,"ACTION")) continue;
 /****************************************************************************/
@@ -2033,7 +2052,7 @@ do_new_notice_ctcp(from, to, str, cmd)
 /**************************** PATCHED by Flier ******************************/
                         /*say("CTCP %s reply from %s: %s", cmd, from,
 				args);*/
-                        if (my_stricmp(cmd,"PING")) {
+                        if (my_stricmp(cmd, "PING") && my_stricmp(cmd, "DCC")) {
 #ifdef WANTANSI
                             say("%sCTCP %s%s reply from %s%s%s: %s",
                                 CmdsColors[COLCTCP].color4,cmd,Colors[COLOFF],
@@ -2042,7 +2061,8 @@ do_new_notice_ctcp(from, to, str, cmd)
                             say("CTCP %s reply from %s: %s",cmd,from,args);
 #endif
                         }
-                        else {
+                        else if (my_stricmp(cmd, "DCC") &&
+                                 my_strnicmp(args, "REJECT ", 7)) {
                             isitme=!my_stricmp(from,get_server_nickname(from_server));
 #ifdef WANTANSI
                             say("%sCTCP %s%s reply from %s%s%s: %s",
