@@ -33,7 +33,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: input.c,v 1.22 2006/04/30 14:15:43 f Exp $
+ * $Id: input.c,v 1.27 2008-10-04 17:37:46 f Exp $
  */
 
 #include "irc.h"
@@ -74,6 +74,9 @@ static struct mb_data mbdata;
 static int mbdata_ok = 0;
 
 /**************************** PATCHED by Flier ******************************/
+static int sz_width = 10; /* how far from right border to shift */
+static int sz_prev_len = 0; /* how many prev. characters to show on shift */
+
 static void ResetNickCompletion() {
     tabnickcompl = NULL;
 }
@@ -93,6 +96,7 @@ cursor_to_input(void)
 		{
 			term_move_cursor(screen->inputdata.cursor_x, screen->inputdata.cursor_y);
 			Debug((3, "cursor_to_input: moving cursor to input for screen %d", screen->screennum));
+			Debug((3, "- x %u; y %u", screen->inputdata.cursor_x, screen->inputdata.cursor_y));
 			cursor_not_in_display();
 			term_flush();
 		}
@@ -327,18 +331,7 @@ input_do_check_prompt(int update)
 	if (current_screen->promptlist)
 		prompt = current_screen->promptlist->prompt;
 	else
-/**************************** PATCHED by Flier ******************************/
-        {
-/****************************************************************************/
-		prompt = input_prompt;
-/**************************** PATCHED by Flier ******************************/
-                /*
-                 * fix bug where old prompt was not replaced in case of
-                 * an empty input_prompt
-                 */
-                if (prompt == NULL) prompt = empty_string;
-        }
-/****************************************************************************/
+		prompt = input_prompt ? input_prompt : empty_string;
 	if (prompt)
 	{
 		if (update != NO_UPDATE)
@@ -349,14 +342,12 @@ input_do_check_prompt(int update)
 			int	args_used;	/* this isn't used here but is
 						 * passed to expand_alias()
 						 */
-#ifndef _Windows
 			if (is_process(get_target_by_refnum(0)))
 			{
 				ptr = get_prompt_by_refnum(0);
 				free_it = 0;
 			}
 			else
-#endif /* _Windows */
 				ptr = expand_alias((u_char *) 0, prompt, empty_string, &args_used, NULL);
 
 			len = strlen(ptr);
@@ -388,10 +379,19 @@ input_check_resized(void)
 	inputdata->old_co   = current_screen->co;
 	
 	inputdata->zone     = current_screen->co;
-	if (inputdata->zone > WIDTH)
-		inputdata->zone -= WIDTH;
-	if (inputdata->zone > WIDTH)
-		inputdata->zone -= WIDTH;
+/**************************** PATCHED by Flier ******************************/
+        if (get_int_var(INPUT_PROMPT_SHIFT_NEW_VAR)) {
+            /* 10% of context seems useful */
+            sz_prev_len = current_screen->inputdata.old_co / 10 + 1;
+            sz_width = 2;
+        }
+        else {
+            sz_prev_len = 0;
+            sz_width = WIDTH;
+        }
+	if (inputdata->zone > sz_width)
+		inputdata->zone -= sz_width;
+/****************************************************************************/
 	return 1;
 }
 
@@ -487,6 +487,24 @@ update_input(update)
 
 		window = column - (column % inputdata->zone);
 		
+/**************************** PATCHED by Flier ******************************/
+                /* window defines how much of the left side of input prompt */
+                /* is not visible */
+                if (column >= inputdata->zone)
+                {
+                    if (column >= 2 * inputdata->zone - sz_prev_len)
+                    {
+                        int len = inputdata->zone - sz_prev_len;
+
+                        window = (column - sz_prev_len) / len * len;
+                    }
+                    else
+                    {
+                        window -= sz_prev_len;
+                    }
+                }
+/****************************************************************************/
+
 		/* Recalculate left_ptr */
 		for (column = ptr = 0; column < window; )
 		{
@@ -527,19 +545,16 @@ update_input(update)
 			if (displayable_unival(unival, display_conv))
 			{
 				column += calc_unival_width(unival);
-/**************************** PATCHED by Flier ******************************/
-                                /*
-                                 * hide password prompts
-                                 */
-                                if (!term_echo && (unival == ' ')) {
-                                    char *space = " ";
-
-                                    memcpy(VisBuf+optr, space, 1);
-                                }
-                                else
-/****************************************************************************/
-				memcpy(VisBuf+optr, buf+iptr, len);
-				optr += len;
+				if (!term_echo && unival == ' ')
+				{
+					memcpy(VisBuf+optr, " ", 1);
+					optr += 1;
+				}
+				else
+				{
+					memcpy(VisBuf+optr, buf+iptr, len);
+					optr += len;
+				}
 				iptr += len;
 			}
 			else
@@ -571,6 +586,9 @@ update_input(update)
 	if (update != NO_UPDATE)
 	{
 #ifdef HAVE_ICONV_OPEN
+/**************************** PATCHED by Flier ******************************/
+            if (display_conv != (iconv_t) (-1))
+/****************************************************************************/
 		iconv_close(display_conv);
 #endif /* HAVE_ICONV_OPEN */
 	}
@@ -835,7 +853,8 @@ input_add_character(key, ptr)
 	static unsigned input_pos=0;
 	size_t retval;
 	
-	u_char *iptr, *optr;
+	iconv_const char *iptr;
+	char *optr;
 	size_t isize, osize;
 	
 	input_buffer[input_pos++] = key;
@@ -851,14 +870,27 @@ re_encode:
 	{
 		return;
 	}
-	iptr = input_buffer;
+	iptr = (iconv_const char *)input_buffer;
 	isize = input_pos;
-	optr = output_buffer;
+	optr = (char *)output_buffer;
 	osize = sizeof output_buffer;
 	
+/**************************** PATCHED by Flier ******************************/
+        if (mbdata.conv_in == NULL) {
+            size_t cnt = isize < osize ? isize : osize;
+
+            strmcpy(output_buffer, input_buffer, cnt + 1);
+            while (cnt > 0) {
+                optr++;
+                cnt--;
+            }
+            retval = 0;
+        }
+        else
+/****************************************************************************/
 	retval = iconv(mbdata.conv_in,
-	               (iconv_const char **)&iptr, &isize,
-	               (char **)&optr, &osize);
+		       (iconv_const char **) &iptr, &isize,
+		       &optr, &osize);
 	
 	if (retval == (size_t)-1)
 	{
@@ -871,6 +903,22 @@ re_encode:
 			}
 			case EILSEQ:
 			{
+/**************************** PATCHED by Flier ******************************/
+                                static time_t lastkeypress = 0;
+
+                                if (lastkeypress == 0) lastkeypress = time(NULL);
+                                else {
+                                    time_t timenow = time(NULL);
+
+                                    if (timenow - lastkeypress > 2) {
+                                        /* Probably bad input encoding, ignore bad bytes! */
+                                        say("Wrong input sequence, resetting");
+                                        input_pos = 0;
+                                        input_buffer[0] = '\0';
+                                        return;
+                                    }
+                                }
+/****************************************************************************/
 				/* User gave a bad byte. Ignore bad bytes! */
 				memmove(input_buffer+1,
 				        input_buffer,
@@ -897,7 +945,6 @@ re_encode:
 /****************************************************************************/
 }
 
-#ifndef _Windows
 /*
  * set_input: sets the input buffer to the given string, discarding whatever
  * was in the input buffer before 
@@ -916,20 +963,15 @@ set_input(str)
 #endif /* HAVE_ICONV_OPEN */
 	for (dest = 0; *str != '\0'; )
 	{
-		if (dest + 8 >= sizeof(converted_input))
-			break;
-		
 /**************************** PATCHED by Flier ******************************/
-                /*
-                 * copy attribute characters/beeps directly so we can
-                 * properly recall previous commands
-                 */
                 if (*str == REV_TOG || *str == UND_TOG || *str == BOLD_TOG ||
                     *str == ALL_OFF || *str == '\007') {
                     converted_input[dest++] = *str++;
                     continue;
                 }
 /****************************************************************************/
+		if (dest + 8 >= sizeof(converted_input))
+			break;
 
 		decode_mb(str, converted_input+dest, &mbdata1);
 		dest += mbdata1.output_bytes;
@@ -938,6 +980,11 @@ set_input(str)
 	converted_input[dest] = '\0';
 
 	set_input_raw(converted_input);
+
+/**************************** PATCHED by Flier ******************************/
+        /* fix memory leak */
+        mbdata_done(&mbdata1);
+/****************************************************************************/
 }
 
 void
@@ -962,7 +1009,6 @@ set_input_raw(str)
 		mbdata_ok = 0;
 	}
 }
-#endif /* _Windows */
 
 /*
  * get_input: returns a pointer to the input buffer.  Changing this will
@@ -972,17 +1018,23 @@ set_input_raw(str)
 char	*
 get_input()
 {
-	char* source = get_input_raw();
+	iconv_const char* source = (iconv_const char*)get_input_raw();
 	
 	/* The input buffer is UTF-8 encoded. Clients will want irc_encoding instead. */
-	
-	static char converted_buffer[INPUT_BUFFER_SIZE];
+	static u_char converted_buffer[INPUT_BUFFER_SIZE];
 
 #ifdef HAVE_ICONV_OPEN
-	iconv_t conv = iconv_open(irc_encoding, "UTF-8");
-	char* dest = converted_buffer;
+	iconv_t conv = (iconv_t) (-1);
+	char* dest = (char *)converted_buffer;
 	size_t left, space;
 
+/**************************** PATCHED by Flier ******************************/
+	if (irc_encoding) conv = iconv_open(irc_encoding, "UTF-8");
+        if (conv == (iconv_t) (-1)) {
+            strmcpy(dest, (char *) source, sizeof(converted_buffer));
+            return(converted_buffer);
+        }
+/****************************************************************************/
 	left = strlen(source);
 	space  = sizeof(converted_buffer);
 	while (*source != '\0')
@@ -990,8 +1042,8 @@ get_input()
 		size_t retval;
 
 		retval = iconv(conv,
-		               (iconv_const char**) &source, &left,
-		                (char **)&dest, &space);
+		               (iconv_const char **) &source, &left,
+		                &dest, &space);
 		if (retval == (size_t)-1)
 		{
 			if (errno == E2BIG)
@@ -1014,7 +1066,7 @@ get_input()
 		}
 	}
 	/* Reset the converter, create a reset-sequence */
-	iconv(conv, NULL, &left, (char **)&dest, &space);
+	iconv(conv, NULL, &left, &dest, &space);
 	
 	/* Ensure null-terminators are where they should be */
 	converted_buffer[sizeof(converted_buffer)-1] = '\0';
@@ -1208,8 +1260,6 @@ set_input_prompt(prompt)
 {
 	if (!prompt)
 	{
-		if (!input_prompt)
-			return;
 		malloc_strcpy(&input_prompt, empty_string);
 	}
 	else
@@ -1300,3 +1350,16 @@ function_curpos(input)
 	malloc_strcpy(&new, pos);
 	return new;
 }
+
+/**************************** PATCHED by Flier ******************************/
+/* set parameters for input prompt shifting */
+void
+set_input_prompt_shift_new(val)
+int val;
+{
+    ScreenInputData* inputdata = &current_screen->inputdata;
+
+    inputdata->old_co = 0;   /* to force recalculation */
+    input_check_resized();
+}
+/****************************************************************************/
