@@ -31,12 +31,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: irc.c,v 1.138 2009-12-21 15:08:26 f Exp $
+ * $Id: irc.c,v 1.138 2021-04-26 20:00:58 t Exp $
  */
 
-#define IRCII_VERSION	"20111230"	/* YYYYMMDD */
+#define IRCII_VERSION	"20210506"	/* YYYYMMDD */
 /**************************** PATCHED by Flier ******************************/
-#define SCROLLZ_VERSION "2.2.1"
+#define SCROLLZ_VERSION "2.3.1"
 /****************************************************************************/
 
 #include "irc.h"
@@ -228,6 +228,7 @@ extern void Logo _((char *, char *, char *));
 extern void CheckTimeMinute _((void));
 extern void CheckCdccTimers _((void));
 extern void InitVars _((void));
+extern void InitOther _((void));
 extern void Reset _((char *, char *, char *));
 extern void SwitchNick _((void));
 extern void CleanUp _((void));
@@ -265,7 +266,7 @@ char  defban;
 char  bold=2;
 char  *DefaultServer=(char *) 0;
 char  *ScrollZstr=(char *) 0;
-char  *ScrollZver="ircII " IRCII_VERSION "+ScrollZ " SCROLLZ_VERSION " (30.12.2011)+Cdcc v" CDCC_VERSION;
+char  *ScrollZver="ircII " IRCII_VERSION "+ScrollZ " SCROLLZ_VERSION " (11.2.2016)+Cdcc v" CDCC_VERSION;
 char  *ScrollZver1=(char *) 0;
 #ifdef EXTRA_STUFF
 char  *EString=(char *) 0;
@@ -472,6 +473,7 @@ int   AwayEncrypt;
 int   BanTime;
 int   StatusLines = DEFAULT_STATUS_LINES - 1;
 int   ShowChan;
+int   NotificationLevel = 0;
 #ifdef ACID
 int   ForceJoin;
 #endif
@@ -606,12 +608,27 @@ char VersionInfo[] = {
     '-',
     '8',
 #endif
+#ifdef INET6
+    ' ',
+    'I',
+    'P',
+    'v',
+    '6',
+#endif
 #ifdef HAVE_GMP
     ' ',
     'F',
     'i',
     'S',
     'H',
+#endif
+#ifdef SZTRACE
+    ' ',
+    'T',
+    'R',
+    'A',
+    'C',
+    'E',
 #endif
     '\0'};
 /*****************************/
@@ -646,6 +663,7 @@ irc_exit(quit)
 		new_stty("sane");
 #endif /* ESIX */
 /**************************** PATCHED by Flier ******************************/
+                term_close();
 #ifdef SZNCURSES
                 endwin();
 #endif /* SZNCURSES */
@@ -1011,6 +1029,12 @@ parse_args(argv, argc)
 					connect_at_startup = 0;
 					break;
 /**************************** PATCHED by Flier ******************************/
+                                case '4':
+                                        set_string_var(DEFAULT_PROTOCOL_VAR, "4");
+                                        break;
+                                case '6':
+					set_string_var(DEFAULT_PROTOCOL_VAR, "6");
+					break;
                                 /* what the hell is this */
 				/*case '-':
 					if (argv[ac])
@@ -1818,6 +1842,7 @@ main(argc, argv, envp)
 
 /**************************** Patched by Flier ******************************/
         InitVars();
+        InitOther();
 /****************************************************************************/
 	init_variables();
 /**************************** Patched by Flier ******************************/
