@@ -51,6 +51,7 @@
 /**************************** PATCHED by Flier ******************************/
 #include "myvars.h"
 #include "whowas.h"
+#include "trace.h"
 
 #include <sys/time.h>
 #include <unistd.h>
@@ -241,6 +242,7 @@ add_channel(channel, server, connected, copy, key, nowho)
 	int	do_add = 0;
 /**************************** PATCHED by Flier ******************************/
         int     resetchan = 0;
+        Window  *tmpwin;
         WhowasChanList *whowaschan;
 /****************************************************************************/
 
@@ -262,14 +264,21 @@ add_channel(channel, server, connected, copy, key, nowho)
                 new_free(&(new->topiclock));
                 ClearBans(new);
                 new_free(&new);
+                Trace(SZ_TRACE_CHANNEL, "unlink channel %s from server %d", channel, server);
             }
             new = whowaschan->channellist;
             new->next = (ChannelList *) 0;
-            if ((new->window = is_bound(channel, server)) == (Window *) 0)
-                new->window = curr_scr_win;
             do_add = 1;
             add_to_list((List **) &server_list[server].chan_list, (List *) new);
             new_free(&whowaschan);
+            if (!IsValidWindow(server, new->window)) {
+                new->window = NULL;
+                Trace(SZ_TRACE_CHANNEL, "invalid window for channel %s, resetting", channel);
+            }
+            Trace(SZ_TRACE_CHANNEL, "channel %s found in whowas cache, window %d (%s) %p", channel,
+                  new->window ? new->window->refnum : -1,
+                  new->window ? EMPTY_STR(new->window->name) : "",
+                  new->window);
         }
         else
 /****************************************************************************/
@@ -284,8 +293,6 @@ add_channel(channel, server, connected, copy, key, nowho)
 		malloc_strcpy(&new->channel, channel);
 		new->mode = 0;
 		new->limit = 0;
-		if ((new->window = is_bound(channel, server)) == (Window *) 0)
-			new->window = curr_scr_win;
 		do_add = 1;
 		add_to_list((List **) &server_list[server].chan_list, (List *) new);
 /**************************** PATCHED by Flier ******************************/
@@ -293,13 +300,30 @@ add_channel(channel, server, connected, copy, key, nowho)
                 new->modelock = (char *) 0;
                 new->topiclock = (char *) 0;
                 new->chanlogfpath = (char *) 0;
+                new->window = NULL;
                 resetchan = 1;
+                Trace(SZ_TRACE_CHANNEL, "add channel %s to server %d", channel, server);
 /****************************************************************************/
 	}
         else
         {
 /**************************** Patched by Flier ******************************/
                 if (!(new->nicks)) resetchan = 1;
+                if (!IsValidWindow(server, new->window)) {
+                    new->window = NULL;
+                    Trace(SZ_TRACE_CHANNEL, "invalid window for channel %s, resetting", channel);
+                }
+                Trace(SZ_TRACE_CHANNEL, "channel %s found on server %d, window %d (%s) %p", channel,
+                      server, new->window ? new->window->refnum : -1,
+                      new->window ? EMPTY_STR(new->window->name) : "",
+                      new->window);
+                if ((tmpwin = is_bound(channel, server))) {
+                    Trace(SZ_TRACE_CHANNEL, "bound to window %d (%s) %p",
+                          tmpwin ? tmpwin->refnum : -1,
+                          tmpwin ? EMPTY_STR(tmpwin->name) : "",
+                          tmpwin);
+                    new->window = tmpwin;
+                }
 /****************************************************************************/
 		if (new->connected != CHAN_LIMBO && new->connected != CHAN_JOINING)
 			yell("--- add_channel: add_channel found channel not CHAN_LIMBO/JOINING: %s", new->channel);
@@ -358,6 +382,24 @@ add_channel(channel, server, connected, copy, key, nowho)
                     new->topicwho = NULL;
                     for (i = 0; i < HASHTABLESIZE; i++) new->nickshash[i] = (struct hashstr *) 0;
                 }
+                if (new->window == NULL) {
+                    if ((tmpwin = is_bound(channel, server))) {
+                        Trace(SZ_TRACE_CHANNEL, "bound to window %d (%s) %p",
+                              tmpwin ? tmpwin->refnum : -1,
+                              tmpwin ? EMPTY_STR(tmpwin->name) : "",
+                              tmpwin);
+                        new->window = tmpwin;
+                    }
+                    else {
+                        Trace(SZ_TRACE_CHANNEL, "linked to current window");
+                        new->window = curr_scr_win;
+                    }
+                }
+                Trace(SZ_TRACE_WINDOW, "channel %s linked to window %d (%s) (%p)",
+                      channel,
+                      new->window ? new->window->refnum : -1,
+                      new->window ? EMPTY_STR(new->window->name) : "",
+                      new->window);
 /****************************************************************************/
 		clear_channel(new);
 	}
@@ -392,6 +434,8 @@ add_channel(channel, server, connected, copy, key, nowho)
 					set_channel_by_refnum(tmp->refnum, channel);
 					new->window = tmp;
 					update_all_status();
+                                        Trace(SZ_TRACE_WINDOW, "adding channel %s", channel);
+                                        TraceWindowInfo(2, tmp);
 					return;
 				}
 				else if (!possible)
@@ -403,10 +447,14 @@ add_channel(channel, server, connected, copy, key, nowho)
 			set_channel_by_refnum(possible->refnum, channel);
 			new->window = possible;
 			update_all_status();
+                        Trace(SZ_TRACE_WINDOW, "adding channel %s", channel);
+                        TraceWindowInfo(2, possible);
 			return;
 		}
 		set_channel_by_refnum(0, channel);
 		new->window = curr_scr_win;
+                Trace(SZ_TRACE_WINDOW, "adding channel %s to current window", channel);
+                TraceWindowInfo(2, curr_scr_win);
 	}
 	update_all_windows();
 }
@@ -563,6 +611,7 @@ ChannelList *add_to_channel(channel, nick, server, oper, halfop, voice, userhost
                 if (userhost &&
                     (whowas = check_whowas_buffer(nick, userhost, channel, 1))) {
                     new = whowas->nicklist;
+                    new_free(&whowas->channel);
                     new_free(&whowas);
                     snprintf(tmpbuf, sizeof(tmpbuf), "%s!%s", nick, userhost);
                     if (!(new->frlist &&
@@ -586,6 +635,8 @@ ChannelList *add_to_channel(channel, nick, server, oper, halfop, voice, userhost
                     new->minuso = tmp ? tmp->minuso : 0;
                     new->plusb = tmp ? tmp->plusb : 0;
                     new->minusb = tmp ? tmp->minusb : 0;
+                    new->plush = tmp ? tmp->plush : 0;
+                    new->minush = tmp ? tmp->minush : 0;
                     new->kick = tmp ? tmp->kick : 0;
                     new->nickc = tmp ? tmp->nickc : 0;
                     new->publics = tmp ? tmp->publics : 0;
@@ -1647,7 +1698,13 @@ free_channel(channel)
         for (nick=(*channel)->nicks;nick;) {
             tmpnick=nick;
             nick=nick->next;
-            add_to_whowas_buffer(tmpnick,(*channel)->channel);
+            if (tmpnick && tmpnick->userhost)
+                add_to_whowas_buffer(tmpnick,(*channel)->channel);
+            else {
+                /* in case of /JOIN -NOWHO #channel */
+                new_free(&tmpnick->nick);
+                new_free(&tmpnick);
+            }
         }
         (*channel)->nicks=(NickList *) 0;
         if (!(add_to_whowas_chan_buffer((*channel)))) {
@@ -1854,6 +1911,8 @@ rename_nick(old_nick, new_nick, server)
 			{
 /**************************** PATCHED by Flier ******************************/
                                 remove_nick_from_hash(chan,tmp);
+                                Trace(SZ_TRACE_NICK, "nick rename %s -> %s in %s",
+                                      old_nick, new_nick, chan->channel);
 /****************************************************************************/
 				new_free(&tmp->nick);
 				malloc_strcpy(&tmp->nick, new_nick);
@@ -2136,6 +2195,9 @@ reconnect_all_channels(server)
 /****************************************************************************/
 
 /**************************** PATCHED by Flier ******************************/
+    Trace(SZ_TRACE_CHANNEL, "reconnecting all channels for server %d", server);
+    TraceChannelInfo(2, server_list[server].chan_list);
+
         /*for (tmp = server_list[server].chan_list; tmp; tmp = tmp->next)*/
         for (tmp = server_list[server].chan_list; tmp; tmp = next)
 /****************************************************************************/
@@ -2177,6 +2239,10 @@ reconnect_all_channels(server)
                 }
 /****************************************************************************/
 	}
+/**************************** PATCHED by Flier ******************************/
+    Trace(SZ_TRACE_CHANNEL, "reconnect done");
+    TraceChannelInfo(2, server_list[server].chan_list);
+/****************************************************************************/
 }
 
 char	*
