@@ -499,7 +499,8 @@ password_sendline(data, line)
 	new_server = atoi(data);
 	set_server_password(new_server, line);
 	connect_to_server(get_server_name(new_server),
-		get_server_port(new_server), get_server_nickname(new_server), -1);
+		get_server_port(new_server), get_server_nickname(new_server),
+		get_server_group(new_server), -1);
 }
 
 /*
@@ -928,9 +929,9 @@ version(from, ArgList)
 		PasteArgs(ArgList, 1);
 /**************************** Patched by Flier ******************************/
 		/*put_it("%s Server %s: %s", numeric_banner(), ArgList[1],*/
-		put_it("%sServer %s: %s", numeric_banner(), ArgList[1],
+                if (ArgList[1]) put_it("%sServer %s: %s", numeric_banner(), ArgList[1], ArgList[0]);
+                else put_it("%sServer %s: %s", numeric_banner(), from, ArgList[0]);
 /****************************************************************************/
-			ArgList[0]);
 	}
 }
 
@@ -1228,6 +1229,9 @@ numbered_command(from, comm, ArgList)
 	case 338:
                 whois_actually(from, ArgList);
                 break;
+	case 378: /* freenode */
+                whois_connecting(from, ArgList);
+                break;
 /****************************************************************************/
 
 	case 321:		/* #define RPL_LISTSTART        321 */
@@ -1693,6 +1697,13 @@ numbered_command(from, comm, ArgList)
                                 CdccTimeWarning();
                         }
                         break;
+		case 346:
+		case 348:
+		case 367:
+                        PasteArgs(ArgList, 0);
+                        if (do_hook(current_numeric, "%s %s", from, *ArgList))
+                            put_it("%s%s", numeric_banner(), ArgList[0]);
+			break;
 /****************************************************************************/
 
 		case 351:		/* #define RPL_VERSION          351 */
