@@ -62,6 +62,7 @@
 #include "list.h"
 #include "status.h"
 #include "myvars.h"
+#include "trace.h"
 
 extern void OnWho _((char *, char *, char *, char *, char *));
 extern void PrintMessage _((char *, char *, char *, int, int));
@@ -324,7 +325,8 @@ p_topic(from, ArgList)
 
             snprintf(tmpbuf3, sizeof(tmpbuf3), "%s topic %s", from, ArgList[0]);
             filepath = OpenCreateFile("ScrollZ.notepad", 1);
-            numurl = GrabURL(ArgList[1], tmpbuf2, filepath, tmpbuf3);
+            numurl = GrabURL(ArgList[1], tmpbuf2, sizeof(tmpbuf2), filepath,
+                             tmpbuf3, empty_string, 0);
         }
         if ((double_ignore(ArgList[0], NULL, IGNORE_CRAP)) == IGNORED) return;
         if ((double_ignore(FromUserHost, ArgList[0], IGNORE_CRAP)) == IGNORED) return;
@@ -992,7 +994,8 @@ p_quit(from, ArgList)
 
 	if (!from)
 		return;
-	flag = double_ignore(from, FromUserHost, IGNORE_CRAP);
+	/*flag = double_ignore(from, FromUserHost, IGNORE_CRAP);*/
+	flag = double_ignore(from, FromUserHost, IGNORE_QUIT);
  	save_message_from();
 	if (flag != IGNORED)
 	{
@@ -1002,6 +1005,8 @@ p_quit(from, ArgList)
 		{
 			message_from(chan, LOG_CRAP);
 /**************************** PATCHED by Flier ******************************/
+                        if ((flag = double_ignore(from, chan, IGNORE_QUIT)) == IGNORED)
+                            goto done;
 			/*if (do_hook(CHANNEL_SIGNOFF_LIST, "%s %s %s", chan, from, Reason))
 				one_prints = 1;*/
                         HandleSplit(Reason,from,chan,&netsplit);
@@ -1112,6 +1117,7 @@ p_quit(from, ArgList)
 /***************************************************************************/
 		}
 	}
+done:
 	message_from((char *) 0, LOG_CURRENT);
 	remove_from_channel((char *) 0, from, parsing_server_index);
 	notify_mark(from, 0, 0);
@@ -1510,15 +1516,24 @@ p_nick(from, ArgList)
 
 	if (!from)
 		return;
-	flag = double_ignore(from, FromUserHost, IGNORE_CRAP);
+	/*flag = double_ignore(from, FromUserHost, IGNORE_CRAP);*/
+	flag = double_ignore(from, FromUserHost, IGNORE_NICK);
 	line = ArgList[0];
 /**************************** Patched by Flier ******************************/
 	/*if (my_stricmp(from, get_server_nickname(parsing_server_index)) == 0) {*/
+        Trace(SZ_TRACE_NICK, "nick change %s -> %s", from, line);
+        Trace(SZ_TRACE_NICK, "SentNick %d   OldNick %s  my nick %s", SentNick,
+              OldNick ? OldNick : empty_string,
+              get_server_nickname(parsing_server_index));
 	if ((SentNick && !my_stricmp(from, OldNick)) ||
             !my_stricmp(from, get_server_nickname(parsing_server_index))) {
 /****************************************************************************/
 		if (parsing_server_index == primary_server)
  			malloc_strcpy(&nickname, line);
+/**************************** Patched by Flier ******************************/
+                Trace(SZ_TRACE_NICK, "nick set server nickname for server %d to %s",
+                      parsing_server_index, line);
+/****************************************************************************/
 		set_server_nickname(parsing_server_index, line);
 		its_me = 1;
 /**************************** Patched by Flier ******************************/
@@ -1930,6 +1945,9 @@ irc2_parse_server(line)
 	if (*end == '\r')
 		*end-- = '\0';
 
+/**************************** Patched by Flier ******************************/
+        Trace(SZ_TRACE_IO, "<-- %d: %s", parsing_server_index, line);
+/****************************************************************************/
 	if (*line == ':')
 	{
 		if (!do_hook(RAW_IRC_LIST, "%s", line + 1))
