@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: screen.c,v 1.37 2006/05/03 16:49:15 f Exp $
+ * $Id: screen.c,v 1.44 2009-07-29 15:52:20 f Exp $
  */
 
 #include "irc.h"
@@ -66,6 +66,7 @@
 
 /**************************** PATCHED by Flier ******************************/
 #include "myvars.h"
+#include "trace.h"
 
 #ifdef WANTANSI
 extern  int  CountAnsi _((char *, int));
@@ -560,10 +561,12 @@ display_bold(flag)
 }
 
 static void
-display_text(str, length)
-	char   *str;
+display_text(ustr, length)
+	char   *ustr;
 	size_t length;
 {
+	iconv_const char *str = (iconv_const char *)ustr;
+
 	if (length > 0)
 	{
 #ifdef HAVE_ICONV_OPEN
@@ -611,7 +614,7 @@ display_text(str, length)
 				else
 				{
 					retval = iconv(converter,
-					               (iconv_const char **)&str, &length,
+					               (iconv_const char **) &str, &length,
 					               &outptr, &outsize);
 				}
 			
@@ -684,7 +687,7 @@ display_text(str, length)
 /**************************** PATCHED by Flier ******************************/
 #ifdef WANTANSI
                                 else if ((*str == '\033') && (len == 1)) {
-                                    int ansi_count;
+                                    int ansi_count = 0;
                                     char *orig_str = str;
 
                                     if (outptr >= OutBuf + sizeof(OutBuf)) {
@@ -1602,9 +1605,12 @@ split_up_line(str)
 		invert_state = 0,
 		underline_state = 0;
 /**************************** PATCHED by Flier ******************************/
+        int     iso2022 = get_int_var(ISO2022_SUPPORT_VAR);
+        int     isiso;
 #ifdef WANTANSI
         int     ansi_count;
 #endif /* WANTANSI */
+        int     word_break_col;
 /****************************************************************************/
 		
 	size_t	len;
@@ -1664,6 +1670,9 @@ split_up_line(str)
 					indent = -1;
 				len = 8 - (col % 8);
 				word_break = pos;
+/**************************** PATCHED by Flier ******************************/
+                                word_break_col = col;
+/****************************************************************************/
 				for (i = 0; i < len; i++)
 					lbuf[pos++] = ' ';
 				col += len;
@@ -1693,15 +1702,32 @@ split_up_line(str)
 			break;
 /**************************** PATCHED by Flier ******************************/
 #ifdef WANTANSI
+                        /* XXX: flier fix for ISO-2022-JP */
                 case '\033':
                         ansi_count = 0;
-                        while (vt100Decode(*ptr)) {
+                        isiso = 0;
+                        while ((c = vt100Decode(*ptr))) {
+                            /* terminate on escape if ISO-2022-JP support is enabled */
+                            if (iso2022 && *ptr == '\033' && ansi_count) break;
+                            if (c > 1) isiso = 1;
                             lbuf[pos++] = *ptr;
                             nd_cnt++;
                             ptr++;
                             ansi_count++;
                         }
-                        if (ansi_count) ptr--;
+                        if (isiso || ansi_count) ptr--;
+                        if (ansi_count && iso2022 && isiso) {
+                            char lbuf2[BIG_BUFFER_SIZE];
+                            char lbuf3[BIG_BUFFER_SIZE];
+
+                            memcpy(lbuf2, lbuf + pos - ansi_count, ansi_count);
+                            lbuf2[ansi_count] = '\0';
+                            pos -= ansi_count;
+                            nd_cnt -= ansi_count;
+
+                            /* FIXME: No array bound checking - very insecure */
+                            decode_mb(lbuf2, lbuf3, &mbdata);
+                        }
                         break;
 #endif /* WANTANSI */
                 case '|':       /* possible word break if time stamp is set to max */
@@ -1714,6 +1740,9 @@ split_up_line(str)
 			if (indent == 0)
 				indent = -1;
 			word_break = pos;
+/**************************** PATCHED by Flier ******************************/
+                        word_break_col = col;
+/****************************************************************************/
 			lbuf[pos++] = *ptr;
 			col++;
 			break;
@@ -1729,7 +1758,14 @@ split_up_line(str)
 				 * be wrappable now when we can
 				 */
 				if (mbdata.num_columns > 1)
+/**************************** PATCHED by Flier ******************************/
+                                {
+/****************************************************************************/
 					word_break = pos;
+/**************************** PATCHED by Flier ******************************/
+                                        word_break_col = col;
+                                }
+/****************************************************************************/
 				
 				pos	   += mbdata.output_bytes;
 				nd_cnt += mbdata.output_bytes - mbdata.num_columns;
@@ -1749,6 +1785,10 @@ split_up_line(str)
 			/* one big long line, no word breaks */
 			if (word_break == 0)
 				word_break = pos - (col - current_screen->co);
+/**************************** PATCHED by Flier ******************************/
+                        else if (word_break_col < current_screen->co / 5)
+                            word_break = pos;
+/****************************************************************************/
 			c = lbuf[word_break];
 			c1 = lbuf[word_break+1];
 			lbuf[word_break] = FULL_OFF;
@@ -1761,6 +1801,11 @@ split_up_line(str)
 			else
 				malloc_strcpy(&temp, &(lbuf[start]));
 			malloc_strcpy(&output[line++], temp);
+/**************************** PATCHED by Flier ******************************/
+                        /* do not overflow the buffer */
+                        if (line >= MAXIMUM_SPLITS)
+                            break;
+/****************************************************************************/
 			lbuf[word_break] = c;
 			lbuf[word_break+1] = c1;
 			start = word_break;
@@ -1927,12 +1972,12 @@ int in_redirect;
  * lastlog the information should be added to, which log the information
  * should be sent to, etc 
  */
-void
+Window *
 add_to_screen(incoming)
 	char	*incoming;
 {
 	int	flag;
-	Window	*tmp;
+	Window	*tmp, *w = NULL;
 	char	buffer[BIG_BUFFER_SIZE + 1];
 /**************************** PATCHED by Flier ******************************/
         char tmpbuf[BIG_BUFFER_SIZE + 1];
@@ -1961,7 +2006,10 @@ add_to_screen(incoming)
 	{
 		puts(incoming);
 		fflush(stdout);
-		return;
+/**************************** PATCHED by Flier ******************************/
+		/*return;*/
+        return NULL;
+/****************************************************************************/
 	}
 	/* Handles output redirection first */
 	if (!in_redirect && current_screen->redirect_name &&
@@ -2013,22 +2061,25 @@ add_to_screen(incoming)
 	{
 		/* FIXME: Do iconv for "incoming" in dumb mode too */
 		add_to_lastlog(curr_scr_win, incoming);
+                w = curr_scr_win;
 		if (do_hook(WINDOW_LIST, "%u %s", curr_scr_win->refnum, incoming))
 			puts(incoming);
   		term_flush();
-		return;
+		return w;
 	}
 	if (in_window_command)
 		update_all_windows();
 	if ((who_level == LOG_CURRENT) && (curr_scr_win->server == from_server))
         {
 		add_to_window(curr_scr_win, incoming);
-		return;
+                w = curr_scr_win;
+		return w;
 	}
 	if (to_window)
 	{
 		add_to_window(to_window, incoming);
-		return;
+                w = to_window;
+		return w;
 	}
 	if (who_from)
 	{
@@ -2039,7 +2090,8 @@ add_to_screen(incoming)
 			if ((chan = lookup_channel(who_from, from_server, CHAN_NOUNLINK)))
 			{
 				add_to_window(chan->window ? chan->window : curr_scr_win, incoming);
-				return;
+                                w = chan->window ? chan->window : curr_scr_win;
+				return w;
 			}
 			if (who_level == LOG_DCC)
 			{
@@ -2048,7 +2100,8 @@ add_to_screen(incoming)
 				if ((chan = lookup_channel(buffer, from_server, CHAN_NOUNLINK)))
 				{
 					add_to_window(chan->window ? chan->window : curr_scr_win, incoming);
-					return;
+                                        w = chan->window ? chan->window : curr_scr_win;
+					return w;
 				}
 			}
 		}
@@ -2066,7 +2119,8 @@ add_to_screen(incoming)
 					my_stricmp(who_from, tmp->query_nick + 1) == 0)))
 				{
 					add_to_window(tmp, incoming);
-					return;
+                                        w = tmp;
+					return w;
 				}
 			}
 			flag = 1;
@@ -2077,7 +2131,8 @@ add_to_screen(incoming)
 					if (find_in_list((List **) &(tmp->nicks), who_from, !USE_WILDCARDS))
 					{
 						add_to_window(tmp, incoming);
-						return;
+                                                w = tmp;
+						return w;
 					}
 				}
 			}
@@ -2090,7 +2145,8 @@ add_to_screen(incoming)
 		    (who_level & tmp->window_level))
 		{
 			add_to_window(tmp, incoming);
-			return;
+                        w = tmp;
+			return w;
 		}
 	}
 	if (from_server == curr_scr_win->server)
@@ -2107,6 +2163,8 @@ add_to_screen(incoming)
 			tmp = curr_scr_win;
 	}
 	add_to_window(tmp, incoming);
+        w = tmp;
+        return w;
 }
 
 /*
@@ -2241,11 +2299,7 @@ new_window()
 	new->status_line[1] = NULL;
 /**************************** PATCHED by Flier ******************************/
         /*new->double_status = 0;*/
-#ifndef ACID
-        new->double_status=1;
-#else
-        new->double_status=2;
-#endif
+        new->double_status = StatusLines;
 	new->status_line[2]=NULL;
 /****************************************************************************/
 	new->top_of_display = 0;
@@ -2670,6 +2724,7 @@ next_line_back(window)
                    that (fixes translation when scrolling backwards) */
 		len = strlen(LogLine->msg);
 		my_str = new_malloc(len + 1);
+                Trace(SZ_TRACE_LASTLOG, "next_line_back %p %s", LogLine, LogLine->msg);
 		bcopy(LogLine->msg, my_str, len);
 		my_str[len] = '\0';
 		TheirLines = split_up_line(my_str);
@@ -2697,21 +2752,37 @@ lastlog_lines(window)
 {
 	Display	*Disp;
 	int num_lines = 0, i;
+        int has_next = 0;
+
+        if (window->top_of_display->next && window->top_of_display->next->line)
+            has_next = 1;
+#ifdef SZTRACE
+        long sztrace = GetTraceLevel();
+
+        /* do not log to trace when counting lastlog lines */
+        SetTraceLevel(0);
+#endif
 
 	(void)next_line_back(window);
 
 	for (i = window->new_scrolled_lines; i--; num_lines++)
 		(void)next_line_back(NULL);
 
-	for (i = 0, Disp = window->top_of_display; i < window->display_size;
-			Disp = Disp->next, i++)
-		if (Disp->linetype)
-			(void)next_line_back(NULL);
+        if (has_next) {
+            for (i = 0, Disp = window->top_of_display; i < window->display_size; Disp = Disp->next, i++)
+                if (Disp->linetype)
+                    (void)next_line_back(NULL);
+        }
 
 	while (next_line_back(NULL))
 		num_lines++;
 
 	Debug((3, "  num_lines ends up as %d", num_lines));
+
+#ifdef SZTRACE
+        SetTraceLevel(sztrace);
+#endif
+
 	return (num_lines);
 }
 
@@ -2727,20 +2798,39 @@ display_lastlog_lines(start, end, window)
 	Display	*Disp;
 	char	*Line;
 	int	i;
+/**************************** PATCHED by Flier ******************************/
+        int     has_next = 0;
+
+        if (window->top_of_display->next && window->top_of_display->next->line)
+            has_next = 1;
+        Trace(SZ_TRACE_LASTLOG, "display lastlog window %d (size %d, scrolled %d): start %d  end %d  has_next %d",
+              window->refnum, window->lastlog_size, window->new_scrolled_lines, start,
+              end, has_next);
+/****************************************************************************/
 
 	(void)next_line_back(window);
 
-	for (i = window->new_scrolled_lines; i--;)
-		(void)next_line_back(NULL);
+	for (i = window->new_scrolled_lines; i--;) {
+		Line = next_line_back(NULL);
+                if (Line) Trace(SZ_TRACE_LASTLOG, "lastlog1 skip %d) %s", i, Line);
+        }
 
  	/* WTF is this? -krys */
-	for (i = 0, Disp = window->top_of_display; i < window->display_size;
-			Disp = Disp->next, i++)
-		if (Disp->linetype)
-			(void)next_line_back(NULL);
+/**************************** PATCHED by Flier ******************************/
+	/*for (i = 0, Disp = window->top_of_display; i < window->display_size;
+			Disp = Disp->next, i++)*/
+        for (i = 0, Disp = window->top_of_display; i < window->display_size && has_next;
+             Disp = Disp->next, i++)
+/****************************************************************************/
+		if (Disp->linetype) {
+			Line = next_line_back(NULL);
+                        if (Line) Trace(SZ_TRACE_LASTLOG, "lastlog2 skip %d) %s", i, Line);
+                }
 
-	for (i = 0; i < start; i++)
-		(void)next_line_back(NULL);
+	for (i = 0; i < start; i++) {
+		Line = next_line_back(NULL);
+                if (Line) Trace(SZ_TRACE_LASTLOG, "lastlog3 skip %d) %s", i, Line);
+        }
 
 	for (; i < end; i++)
 	{
@@ -2749,6 +2839,7 @@ display_lastlog_lines(start, end, window)
 		term_move_cursor(0, window->top + window->menu.lines +
 			window->scrolled_lines - i - 1);
 		rite(window, Line, 0, 0, 1, 0);
+                Trace(SZ_TRACE_LASTLOG, "lastlog4 display %d) %s", i, Line);
 	}
 }
 
@@ -2764,6 +2855,7 @@ scrollback_backwards_lines(ScrollDist)
 	int totallines;
 
 	Debug((3, "scrollback_backwards_lines(%d)", ScrollDist));
+        Trace(SZ_TRACE_LASTLOG, "scrollback_backwards_lines %d", ScrollDist);
 	window = curr_scr_win;
 	if (!window->scrolled_lines && !window->scroll)
 	{
@@ -2772,10 +2864,13 @@ scrollback_backwards_lines(ScrollDist)
 	}
 	totallines = lastlog_lines(window);
 	Debug((3, "totallines = %d, scrolled_lines = %d", totallines, window->scrolled_lines));
+        Trace(SZ_TRACE_LASTLOG, "totallines = %d, scrolled_lines = %d", totallines,
+              window->scrolled_lines);
 	if (ScrollDist + window->scrolled_lines > totallines)
 	{
 		ScrollDist = totallines - window->scrolled_lines;
 		Debug((3, "  adjusting ScrollDist to %d", ScrollDist));
+                Trace(SZ_TRACE_LASTLOG, "adjusting ScrollDist to %d", ScrollDist);
 	}
 	if (ScrollDist == 0)
 	{
@@ -2788,6 +2883,10 @@ scrollback_backwards_lines(ScrollDist)
 	    window->top + window->menu.lines,
 	    window->top + window->menu.lines + window->display_size - 1,
 	    -ScrollDist));
+        Trace(SZ_TRACE_LASTLOG, "going to term_scroll(%d, %d, %d)",
+                window->top + window->menu.lines,
+                window->top + window->menu.lines + window->display_size - 1,
+                -ScrollDist);
 	term_scroll(window->top + window->menu.lines,
 	    window->top + window->menu.lines + window->display_size - 1,
 	    -ScrollDist);
@@ -2796,16 +2895,26 @@ scrollback_backwards_lines(ScrollDist)
 	    window->scrolled_lines,
 	    window->new_scrolled_lines,
 	    window->display_size));
+        Trace(SZ_TRACE_LASTLOG, "scrolled_lines %d, new_scrolled_lines %d, display_size %d",
+              window->scrolled_lines,
+              window->new_scrolled_lines,
+              window->display_size);
 
 	Debug((3, "going to display_lastlog_lines(%d, %d, %s)",
 	    window->scrolled_lines - ScrollDist,
 	    window->scrolled_lines,
 	    window->name));
+        Trace(SZ_TRACE_LASTLOG, "going to display_lastlog_lines(%d, %d, %d)",
+              window->scrolled_lines - ScrollDist,
+              window->scrolled_lines,
+              window->refnum);
 	display_lastlog_lines(window->scrolled_lines - ScrollDist,
 	    window->scrolled_lines,
 	    window);
 	cursor_not_in_display();
 	update_input(UPDATE_JUST_CURSOR);
+	window->update |= UPDATE_STATUS;
+	update_window_status(window, 0);
 }
 
 static	void
@@ -2815,6 +2924,7 @@ scrollback_forwards_lines(ScrollDist)
 	Window	*window;
 
 	Debug((3, "scrollback_forward_lines(%d)", ScrollDist));
+        Trace(SZ_TRACE_LASTLOG, "scrollback_forward_lines %d", ScrollDist);
 	window = curr_scr_win;
 	if (!window->scrolled_lines)
 	{
@@ -2825,11 +2935,16 @@ scrollback_forwards_lines(ScrollDist)
 		ScrollDist = window->scrolled_lines;
 
 	Debug((3, "scrolled_lines = %d", window->scrolled_lines));
+        Trace(SZ_TRACE_LASTLOG, "scrolled_lines %d", window->scrolled_lines);
 	window->scrolled_lines -= ScrollDist;
 	Debug((3, "going to term_scroll(%d, %d, %d)",
 	    window->top + window->menu.lines,
 	    window->top + window->menu.lines + window->display_size - 1,
 	    ScrollDist));
+        Trace(SZ_TRACE_LASTLOG, "going to term_scroll(%d, %d, %d)",
+              window->top + window->menu.lines,
+              window->top + window->menu.lines + window->display_size - 1,
+              ScrollDist);
 	term_scroll(window->top + window->menu.lines,
 	    window->top + window->menu.lines + window->display_size - 1,
 	    ScrollDist);
@@ -2838,6 +2953,10 @@ scrollback_forwards_lines(ScrollDist)
 	    window->scrolled_lines,
 	    window->new_scrolled_lines,
 	    window->display_size));
+        Trace(SZ_TRACE_LASTLOG, "scrolled_lines %d, new_scrolled_lines %d, display_size %d",
+              window->scrolled_lines,
+              window->new_scrolled_lines,
+              window->display_size);
 
 	if (window->scrolled_lines < window->display_size)
 		redraw_window(window,
@@ -2847,6 +2966,10 @@ scrollback_forwards_lines(ScrollDist)
 	    window->scrolled_lines - window->display_size,
 	    window->scrolled_lines - window->display_size + ScrollDist,
 	    window->name));
+        Trace(SZ_TRACE_LASTLOG, "going to display_lastlog_lines(%d, %d, %d)",
+              window->scrolled_lines - window->display_size,
+              window->scrolled_lines - window->display_size + ScrollDist,
+              window->refnum);
 	display_lastlog_lines(window->scrolled_lines - window->display_size,
 	    window->scrolled_lines - window->display_size + ScrollDist,
 	    window);
@@ -2861,6 +2984,8 @@ scrollback_forwards_lines(ScrollDist)
 		else
 			hold_mode(window, OFF, 0);
 	}
+	window->update |= UPDATE_STATUS;
+	update_window_status(window, 0);
 }
 
 /*
@@ -2920,6 +3045,8 @@ scrollback_end(key, ptr)
 			hold_mode(window, ON, 1);
 		else
 			hold_mode(window, OFF, 0);
+		window->update |= UPDATE_STATUS;
+		update_window_status(window, 0);
 	}
 }
 
@@ -2963,6 +3090,8 @@ scrollback_start(key, ptr)
 			hold_mode(window, ON, 1);
 		else
 			hold_mode(window, OFF, 0);
+		window->update |= UPDATE_STATUS;
+		update_window_status(window, 0);
 	}
 }
 
