@@ -1347,14 +1347,13 @@ status_server(window)
 
 	if (connected_to_server != 1)
 	{
-		if (window->server != -1)
+		if (is_server_ok(window->server))
 		{
 			if (server_format)
 			{
 				name = get_server_name(window->server);
 				rest = (char *) index(name, '.');
-				if (rest != NULL && my_strnicmp(name, "irc", 3) != 0 &&
-				    my_strnicmp(name, "icb", 3) != 0)
+				if (rest != NULL && my_strnicmp(name, "irc", 3) != 0)
 				{
 					if (is_number(name))
 						snprintf(lbuf, sizeof lbuf, server_format, name);
@@ -1788,7 +1787,7 @@ status_away(window)
 			malloc_strcpy(&ptr, text);
 		else
 			malloc_strcpy(&ptr, empty_string);*/
-                if (server_list[window->server].away && away_format) {
+                if ((window->server != -1) && server_list[window->server].away && away_format) {
                     snprintf(buf, sizeof(buf), "%d", AwayMsgNum);
                     snprintf(locbuf, sizeof(locbuf), away_format, buf);
                     malloc_strcpy(&ptr, locbuf);
