@@ -102,16 +102,22 @@ static  void    SetDCCHost _((char *));
 static  void    SetDCCPorts _((char *));
 static	void	Cnotifystring _((char *));
 static  void    SetAwayFile _((char *));
-void    SetStampFormat _((char *));
+void            SetStampFormat _((char *));
+static  void    SetSSLPriority _((char *));
+static  void    SetSSLCAFile _((char *));
 static  void    SetStatusLines _((int));
 static  void    SetUsername _((char *));
+static  void    SetURLBufferSize _((int));
 
 extern  void    RedrawAll _((void));
 extern  void    UpdateFloodUsers _((void));
+extern  void    SetTrace _((char *));
+extern  void    SetNotificationLevel _((char *));
 
 extern  int     DCCLowPort;
 extern  int     DCCHighPort;
 extern	char	*CelerityNtfy;
+extern  int     URLnum;
 /****************************************************************************/
 
 /*
@@ -165,6 +171,9 @@ IrcVariable irc_variable[] =
 /****************************************************************************/
 	{ "DEBUG",			INT_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "DECRYPT_PROGRAM",		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
+/**************************** PATCHED by Flier ******************************/
+        { "DEFAULT_PROTOCOL",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+/****************************************************************************/
 	{ "DISPLAY",			BOOL_TYPE_VAR,	DEFAULT_DISPLAY, NULL, NULL, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
 	{ "DISPLAY_ANSI",		BOOL_TYPE_VAR,	DEFAULT_DISPLAY_ANSI, NULL, RedrawAll, 0, 0 },
@@ -210,6 +219,9 @@ IrcVariable irc_variable[] =
 	{ "INPUT_ALIASES",		BOOL_TYPE_VAR,	DEFAULT_INPUT_ALIASES, NULL, NULL, 0, 0 },
 	{ "INPUT_ENCODING",		STR_TYPE_VAR,	0, NULL, set_input_encoding, 0, 0 },
 	{ "INPUT_PROMPT",		STR_TYPE_VAR,	0, NULL, set_input_prompt, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+	{ "INPUT_PROMPT_SHIFT_NEW",	BOOL_TYPE_VAR,	DEFAULT_INPUT_PROMPT_SHIFT_NEW, 0, set_input_prompt_shift_new, 0, 0 },
+/****************************************************************************/
 	{ "INPUT_PROTECTION",		BOOL_TYPE_VAR,	DEFAULT_INPUT_PROTECTION, NULL, input_warning, 0, 0 },
 	{ "INSERT_MODE",		BOOL_TYPE_VAR,	DEFAULT_INSERT_MODE, NULL, update_all_status, 0, 0 },
 	{ "INVERSE_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_INVERSE_VIDEO, NULL, NULL, 0, 0 },
@@ -243,6 +255,9 @@ IrcVariable irc_variable[] =
 	{ "MINIMUM_SERVERS",		INT_TYPE_VAR,	DEFAULT_MINIMUM_SERVERS, NULL, NULL, 0, VF_NODAEMON },
 	{ "MINIMUM_USERS",		INT_TYPE_VAR,	DEFAULT_MINIMUM_USERS, NULL, NULL, 0, VF_NODAEMON },
 	{ "NETSPLIT_TIME",		INT_TYPE_VAR,	DEFAULT_NETSPLIT_TIME, NULL, NULL, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+	{ "NOTIFICATION_LEVEL",		STR_TYPE_VAR,	0, NULL, SetNotificationLevel, 0, 0 },
+/****************************************************************************/
 	{ "NOTIFY_HANDLER",		STR_TYPE_VAR, 	0, 0, set_notify_handler, 0, 0 },
 	{ "NOTIFY_LEVEL",		STR_TYPE_VAR,	0, NULL, set_notify_level, 0, 0 },
 	{ "NOTIFY_ON_TERMINATION",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_ON_TERMINATION, NULL, NULL, 0, VF_NODAEMON },
@@ -280,6 +295,9 @@ IrcVariable irc_variable[] =
 	{ "SHOW_STATUS_ALL",		BOOL_TYPE_VAR,	DEFAULT_SHOW_STATUS_ALL, NULL, update_all_status, 0, 0 },
 	{ "SHOW_WHO_HOPCOUNT", 		BOOL_TYPE_VAR,	DEFAULT_SHOW_WHO_HOPCOUNT, NULL, NULL, 0, 0 },
 /**************************** Patched by Flier ******************************/
+        { "SSL_CA_FILE",                STR_TYPE_VAR,   0, NULL, SetSSLCAFile, 0, 0 },
+        { "SSL_PRIORITY_STRING",        STR_TYPE_VAR,   0, NULL, SetSSLPriority, 0, 0 },
+        { "SSL_VERIFY_CERTIFICATE",     BOOL_TYPE_VAR,  DEFAULT_SSL_VERIFY_CERTIFICATE, NULL, NULL, 0, 0 },
 	{ "STAMP_FORMAT",		STR_TYPE_VAR,	0, NULL, SetStampFormat, 0, 0 },
 /****************************************************************************/
 	{ "STATUS_AWAY",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -342,10 +360,12 @@ IrcVariable irc_variable[] =
 	{ "TAB",			BOOL_TYPE_VAR,	DEFAULT_TAB, NULL, NULL, 0, 0 },
 	{ "TAB_MAX",			INT_TYPE_VAR,	DEFAULT_TAB_MAX, NULL, NULL, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
+        { "TRACE",			STR_TYPE_VAR, 	0, NULL, SetTrace, 0, 0 },
         { "TRUNCATE_PUBLIC_CHANNEL",	BOOL_TYPE_VAR, 	DEFAULT_TRUNCATE_PUBLIC_CHANNEL, NULL, NULL, 0, 0 },
 /****************************************************************************/
 	{ "UNDERLINE_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_UNDERLINE_VIDEO, NULL, NULL, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
+        { "URL_BUFFER_SIZE_VAR",		INT_TYPE_VAR,	30, NULL, SetURLBufferSize, 0, 0 },
         { "USERNAME",		STR_TYPE_VAR,	0, NULL, SetUsername, 0, 0 },
 /****************************************************************************/
 	{ "USER_INFORMATION", 		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
@@ -385,6 +405,9 @@ init_variables()
 	set_string_var(HIGHLIGHT_CHAR_VAR, DEFAULT_HIGHLIGHT_CHAR);
 	set_string_var(HISTORY_FILE_VAR, DEFAULT_HISTORY_FILE);
 	set_string_var(LASTLOG_LEVEL_VAR, DEFAULT_LASTLOG_LEVEL);
+/**************************** PATCHED by Flier ******************************/
+        set_string_var(NOTIFICATION_LEVEL_VAR, DEFAULT_NOTIFICATION_LEVEL);
+/****************************************************************************/
 	set_string_var(NOTIFY_HANDLER_VAR, DEFAULT_NOTIFY_HANDLER);
 	set_string_var(NOTIFY_LEVEL_VAR, DEFAULT_NOTIFY_LEVEL);
 	set_string_var(RATE_LIMIT_JOIN_VAR, DEFAULT_RATE_LIMIT_JOIN);
@@ -1119,4 +1142,67 @@ char *new_username;
     if (new_username) strmcpy(username, new_username, NAME_LEN);
     else strmcpy(username, empty_string, NAME_LEN);
 }
+
+void SetSSLPriority(priority)
+char *priority;
+{
+#if defined(HAVE_SSL)
+    const char *errpos;
+    char tmpbuf[mybufsize / 4];
+    char tmpbuf2[mybufsize / 4];
+    gnutls_session_t session;
+
+    gnutls_init(&session, GNUTLS_CLIENT);
+    if (gnutls_priority_set_direct(session, priority, &errpos) != GNUTLS_E_SUCCESS) {
+        strcpy(tmpbuf, "Invalid priority string");
+        if (errpos) {
+            sprintf(tmpbuf2, " at position %d", errpos - priority);
+            strcat(tmpbuf, tmpbuf2);
+        }
+        say("%s", tmpbuf);
+        set_string_var(SSL_PRIORITY_STRING_VAR, NULL);
+    }
+    else set_string_var(SSL_PRIORITY_STRING_VAR, priority);
+    gnutls_deinit(session);
+#else
+    say("SSL_PRIORITY_STRING has no effect - this version was compiled without GnuTLS support");
+#endif
+}
+
+void SetSSLCAFile(filepath)
+char *filepath;
+{
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
+    FILE *fpx;
+
+    if (filepath) {
+        if ((fpx = fopen(filepath, "ro"))) {
+            fclose(fpx);
+            set_string_var(SSL_CA_FILE_VAR, filepath);
+            return;
+        }
+    }
+    set_string_var(SSL_CA_FILE_VAR, NULL);
+#else
+    say("SSL_CA_FILE has no effect - this version was compiled without GnuTLS support");
+#endif
+}
+
+static void SetURLBufferSize(value)
+int value;
+{
+    struct urlstr *tmpurl;
+
+    if (value < 1) value = 1;
+
+    /* delete as many URLs as needed to fit the new buffer size */
+    while (URLnum > get_int_var(URL_BUFFER_SIZE_VAR)) {
+        tmpurl = urllist;
+        urllist = tmpurl->next;
+        new_free(&(tmpurl->urls));
+        new_free(&(tmpurl->source));
+        new_free(&tmpurl);
+        URLnum--;
+    }
+}
 /****************************************************************************/
